@@ -1,18 +1,21 @@
-// * This file is part of the COLOBOT source code
-// * Copyright (C) 2001-2008, Daniel ROUX & EPSITEC SA, www.epsitec.ch
-// *
-// * This program is free software: you can redistribute it and/or modify
-// * it under the terms of the GNU General Public License as published by
-// * the Free Software Foundation, either version 3 of the License, or
-// * (at your option) any later version.
-// *
-// * This program is distributed in the hope that it will be useful,
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// * GNU General Public License for more details.
-// *
-// * You should have received a copy of the GNU General Public License
-// * along with this program. If not, see  http://www.gnu.org/licenses/.
+/*
+ * This file is part of the Colobot: Gold Edition source code
+ * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see http://gnu.org/licenses
+ */
 ////////////////////////////////////////////////////////////////////
 /**
  * \file CBot.h
@@ -25,20 +28,13 @@
 #include "CBotDll.h"                    // public definitions
 #include "CBotToken.h"                  // token management
 
-#define    STACKRUN    true             /// \def return execution directly on a suspended routine
-#define    STACKMEM    true             /// \def preserve memory for the execution stack
+#define    STACKRUN    1                /// \def return execution directly on a suspended routine
+#define    STACKMEM    1                /// \def preserve memory for the execution stack
 #define    MAXSTACK    990              /// \def stack size reserved
 
 #define    EOX         (reinterpret_cast<CBotStack*>(-1))   /// \def tag special condition
 
 
-// fix for MSVC instruction __asm int 3 (setting a trap)
-#if defined(__MINGW32__) || defined(__GNUC__)
-#define ASM_TRAP()    asm("int $3");
-#else
-#define ASM_TRAP()    __asm int 3;
-#endif
-
 /////////////////////////////////////////////////////////////////////
 // forward declaration
 
@@ -1520,7 +1516,7 @@ extern bool ReadType(FILE* pf, CBotTypResult& type);
 
 extern float GetNumFloat( const char* p );
 
-#if    false
+#if 0
 extern void DEBUG( const char* text, int val, CBotStack* pile );
 #endif
 
