@@ -1,23 +1,28 @@
-// * This file is part of the COLOBOT source code
-// * Copyright (C) 2001-2008, Daniel ROUX & EPSITEC SA, www.epsitec.ch
-// *
-// * This program is free software: you can redistribute it and/or modify
-// * it under the terms of the GNU General Public License as published by
-// * the Free Software Foundation, either version 3 of the License, or
-// * (at your option) any later version.
-// *
-// * This program is distributed in the hope that it will be useful,
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// * GNU General Public License for more details.
-// *
-// * You should have received a copy of the GNU General Public License
-// * along with this program. If not, see  http://www.gnu.org/licenses/.
+/*
+ * This file is part of the Colobot: Gold Edition source code
+ * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see http://gnu.org/licenses
+ */
 
 //Management of the stack
 
 
 #include "CBot.h"
+
+#include <cassert>
 #include <cstdlib>
 #include <cstring>
 
@@ -80,12 +85,12 @@ CBotStack* CBotStack::FirstStack()
 CBotStack::CBotStack(CBotStack* ppapa)
 {
     // constructor must exist or the destructor is never called!
-    ASM_TRAP();
+    assert(0);
 }
 
 CBotStack::~CBotStack()
 {
-    ASM_TRAP();    // use Delete () instead
+    assert(0);    // use Delete () instead
 }
 
 void CBotStack::Delete()
@@ -693,7 +698,7 @@ void CBotStack::AddVar(CBotVar* pVar)
     *pp = pVar;                    // added after
 
 #ifdef    _DEBUG
-    if ( pVar->GetUniqNum() == 0 ) ASM_TRAP();
+    if ( pVar->GetUniqNum() == 0 ) assert(0);
 #endif
 }
 
@@ -786,7 +791,7 @@ void CBotStack::GetRunPos(const char* &FunctionName, int &start, int &end)
     while (p->m_next != NULL)
     {
         if ( p->m_instr != NULL ) instr = p->m_instr;
-        if ( p->m_bFunc == 1 ) funct = p->m_instr;
+        if ( p->m_bFunc == 1 && p->m_instr != NULL ) funct = p->m_instr;
         if ( p->m_next->m_prog != prog ) break ;
 
         if (p->m_next2 && p->m_next2->m_state != 0) p = p->m_next2 ;
@@ -794,7 +799,7 @@ void CBotStack::GetRunPos(const char* &FunctionName, int &start, int &end)
     }
 
     if ( p->m_instr != NULL ) instr = p->m_instr;
-    if ( p->m_bFunc == 1 ) funct = p->m_instr;
+    if ( p->m_bFunc == 1 && p->m_instr != NULL ) funct = p->m_instr;
 
     if ( funct == NULL ) return;
 
@@ -805,7 +810,7 @@ void CBotStack::GetRunPos(const char* &FunctionName, int &start, int &end)
 
     t = instr->GetToken();
     start = t->GetStart();
-    end      = t->GetEnd();
+    end   = t->GetEnd();
 }
 
 CBotVar* CBotStack::GetStackVars(const char* &FunctionName, int level)
@@ -823,16 +828,19 @@ CBotVar* CBotStack::GetStackVars(const char* &FunctionName, int level)
         if (p->m_next2 && p->m_next2->m_state != 0) p = p->m_next2 ;
         else                                        p = p->m_next;
     }
+    // Now p is the highest element on the stack
 
 
     // descends upon the elements of block
     while ( p != NULL && !p->m_bBlock )    p = p->m_prev;
+    // Now p is on the beggining of the top block (with local variables)
 
     while ( p != NULL && level++ < 0 )
     {
         p = p->m_prev;
         while ( p != NULL && !p->m_bBlock )    p = p->m_prev;
     }
+    // Now p is on the block "level"
 
     if ( p == NULL ) return NULL;
 
@@ -1091,7 +1099,7 @@ bool CBotVar::RestoreState(FILE* pf, CBotVar* &pVar)
             }
             break;
         default:
-            ASM_TRAP();
+            assert(0);
         }
 
         if ( pPrev != NULL ) pPrev->m_next = pNew;
@@ -1388,7 +1396,7 @@ void CBotCStack::AddVar(CBotVar* pVar)
     *pp = pVar;                    // added after
 
 #ifdef    _DEBUG
-    if ( pVar->GetUniqNum() == 0 ) ASM_TRAP();
+    if ( pVar->GetUniqNum() == 0 ) assert(0);
 #endif
 }
 
