@@ -1,36 +1,46 @@
-// * This file is part of the COLOBOT source code
-// * Copyright (C) 2001-2008, Daniel ROUX & EPSITEC SA, www.epsitec.ch
-// * Copyright (C) 2012, Polish Portal of Colobot (PPC)
-// *
-// * This program is free software: you can redistribute it and/or modify
-// * it under the terms of the GNU General Public License as published by
-// * the Free Software Foundation, either version 3 of the License, or
-// * (at your option) any later version.
-// *
-// * This program is distributed in the hope that it will be useful,
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// * GNU General Public License for more details.
-// *
-// * You should have received a copy of the GNU General Public License
-// * along with this program. If not, see  http://www.gnu.org/licenses/.
+/*
+ * This file is part of the Colobot: Gold Edition source code
+ * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see http://gnu.org/licenses
+ */
 
 #include "common/config.h"
 
 #include "app/app.h"
 
+#include "app/controller.h"
+#include "app/input.h"
 #include "app/system.h"
 
 #include "common/logger.h"
 #include "common/iman.h"
 #include "common/image.h"
 #include "common/key.h"
+#include "common/pathman.h"
 #include "common/stringutils.h"
+#include "common/resources/resourcemanager.h"
 
 #include "graphics/engine/modelmanager.h"
+#include "graphics/core/nulldevice.h"
 #include "graphics/opengl/gldevice.h"
+#include "graphics/opengl/gl21device.h"
+#include "graphics/opengl/gl33device.h"
 
 #include "object/robotmain.h"
+#include "object/objman.h"
 
 #ifdef OPENAL_SOUND
     #include "sound/oalsound/alsound.h"
@@ -44,7 +54,6 @@
 
 #include <stdlib.h>
 #include <libintl.h>
-#include <unistd.h>
 #include <getopt.h>
 #include <localename.h>
 
@@ -97,22 +106,24 @@ CApplication::CApplication()
 {
     m_private       = new ApplicationPrivate();
     m_iMan          = new CInstanceManager();
+    m_pathManager   = new CPathManager();
     m_objMan        = new CObjectManager();
     m_eventQueue    = new CEventQueue();
     m_profile       = new CProfile();
+    m_input         = new CInput();
 
-    m_engine    = nullptr;
-    m_device    = nullptr;
-    m_modelManager = nullptr;
-    m_robotMain = nullptr;
-    m_sound     = nullptr;
+    m_engine        = nullptr;
+    m_device        = nullptr;
+    m_modelManager  = nullptr;
+    m_controller    = nullptr;
+    m_sound         = nullptr;
 
-    m_exitCode  = 0;
-    m_active    = false;
-    m_debugModes = 0;
-    m_customDataPath = false;
+    m_exitCode      = 0;
+    m_active        = false;
+    m_debugModes    = 0;
+    m_restart       = false;
 
-    m_windowTitle = "COLOBOT";
+    m_windowTitle = "Colobot: Gold Edition";
 
     m_simulationSuspended = false;
 
@@ -143,35 +154,18 @@ CApplication::CApplication()
 
     m_mouseMode = MOUSE_SYSTEM;
 
-    m_kmodState = 0;
-    m_mouseButtonsState = 0;
-    m_trackedKeys = 0;
-
-    m_dataPath = COLOBOT_DEFAULT_DATADIR;
-    m_langPath = COLOBOT_I18N_DIR;
-    m_texPackPath = "";
-
     m_runSceneName = "";
     m_runSceneRank = 0;
 
+    m_sceneTest = false;
+    m_headless = false;
+    m_resolutionOverride = false;
+
     m_language = LANGUAGE_ENV;
 
     m_lowCPU = true;
 
-    m_protoMode = false;
-
-    for (int i = 0; i < DIR_MAX; ++i)
-        m_standardDataDirs[i] = nullptr;
-
-    m_standardDataDirs[DIR_AI]       = "ai";
-    m_standardDataDirs[DIR_FONT]     = "fonts";
-    m_standardDataDirs[DIR_HELP]     = "help";
-    m_standardDataDirs[DIR_ICON]     = "icons";
-    m_standardDataDirs[DIR_LEVEL]    = "levels";
-    m_standardDataDirs[DIR_MODEL]    = "models";
-    m_standardDataDirs[DIR_MUSIC]    = "music";
-    m_standardDataDirs[DIR_SOUND]    = "sounds";
-    m_standardDataDirs[DIR_TEXTURE]  = "textures";
+    m_graphics = "opengl";
 }
 
 CApplication::~CApplication()
@@ -179,6 +173,9 @@ CApplication::~CApplication()
     delete m_private;
     m_private = nullptr;
 
+    delete m_input;
+    m_input = nullptr;
+
     delete m_objMan;
     m_objMan = nullptr;
 
@@ -188,6 +185,9 @@ CApplication::~CApplication()
     delete m_profile;
     m_profile = nullptr;
 
+    delete m_pathManager;
+    m_pathManager = nullptr;
+
     delete m_iMan;
     m_iMan = nullptr;
 
@@ -210,12 +210,6 @@ CEventQueue* CApplication::GetEventQueue()
 CSoundInterface* CApplication::GetSound()
 {
     return m_sound;
-
-    for (int i = 0; i < PCNT_MAX; ++i)
-    {
-        GetSystemUtils()->DestroyTimeStamp(m_performanceCounters[i][0]);
-        GetSystemUtils()->DestroyTimeStamp(m_performanceCounters[i][1]);
-    }
 }
 
 ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
@@ -225,13 +219,17 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
         OPT_HELP = 1,
         OPT_DEBUG,
         OPT_RUNSCENE,
+        OPT_SCENETEST,
         OPT_LOGLEVEL,
         OPT_LANGUAGE,
-        OPT_DATADIR,
         OPT_LANGDIR,
-        OPT_TEXPACK,
+        OPT_DATADIR,
+        OPT_SAVEDIR,
+        OPT_MOD,
         OPT_VBO,
-        OPT_PROTO
+        OPT_RESOLUTION,
+        OPT_HEADLESS,
+        OPT_DEVICE
     };
 
     option options[] =
@@ -239,13 +237,17 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
         { "help", no_argument, nullptr, OPT_HELP },
         { "debug", required_argument, nullptr, OPT_DEBUG },
         { "runscene", required_argument, nullptr, OPT_RUNSCENE },
+        { "scenetest", no_argument, nullptr, OPT_SCENETEST },
         { "loglevel", required_argument, nullptr, OPT_LOGLEVEL },
         { "language", required_argument, nullptr, OPT_LANGUAGE },
-        { "datadir", required_argument, nullptr, OPT_DATADIR },
         { "langdir", required_argument, nullptr, OPT_LANGDIR },
-        { "texpack", required_argument, nullptr, OPT_TEXPACK },
+        { "datadir", required_argument, nullptr, OPT_DATADIR },
+        { "savedir", required_argument, nullptr, OPT_SAVEDIR },
+        { "mod", required_argument, nullptr, OPT_MOD },
         { "vbo", required_argument, nullptr, OPT_VBO },
-        { "proto", no_argument, nullptr, OPT_PROTO },
+        { "resolution", required_argument, nullptr, OPT_RESOLUTION },
+        { "headless", no_argument, nullptr, OPT_HEADLESS },
+        { "graphics", required_argument, nullptr, OPT_DEVICE },
         { nullptr, 0, nullptr, 0}
     };
 
@@ -273,19 +275,23 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
             case OPT_HELP:
             {
                 GetLogger()->Message("\n");
-                GetLogger()->Message("Colobot %s (%s)\n", COLOBOT_CODENAME, COLOBOT_VERSION);
+                GetLogger()->Message("%s\n", COLOBOT_FULLNAME);
                 GetLogger()->Message("\n");
                 GetLogger()->Message("List of available options:\n");
                 GetLogger()->Message("  -help               this help\n");
                 GetLogger()->Message("  -debug modes        enable debug modes (more info printed in logs; see code for reference of modes)\n");
                 GetLogger()->Message("  -runscene sceneNNN  run given scene on start\n");
+                GetLogger()->Message("  -scenetest          win every mission right after it's loaded\n");
                 GetLogger()->Message("  -loglevel level     set log level to level (one of: trace, debug, info, warn, error, none)\n");
-                GetLogger()->Message("  -language lang      set language (one of: en, de, fr, pl)\n");
-                GetLogger()->Message("  -datadir path       set custom data directory path\n");
+                GetLogger()->Message("  -language lang      set language (one of: en, de, fr, pl, ru)\n");
                 GetLogger()->Message("  -langdir path       set custom language directory path\n");
-                GetLogger()->Message("  -texpack path       set path to custom texture pack\n");
+                GetLogger()->Message("  -datadir path       set custom data directory path\n");
+                GetLogger()->Message("  -savedir path       set custom save directory path (must be writable)\n");
+                GetLogger()->Message("  -mod path           load datadir mod from given path\n");
                 GetLogger()->Message("  -vbo mode           set OpenGL VBO mode (one of: auto, enable, disable)\n");
-                GetLogger()->Message("  -proto              show prototype levels\n");
+                GetLogger()->Message("  -resolution WxH     set resolution\n");
+                GetLogger()->Message("  -headless           headless mode - disables graphics, sound and user interaction\n");
+                GetLogger()->Message("  -graphics           changes graphics device (defaults to opengl)\n");
                 return PARSE_ARGS_HELP;
             }
             case OPT_DEBUG:
@@ -316,6 +322,11 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
                 GetLogger()->Info("Running scene '%s%d' on start\n", m_runSceneName.c_str(), m_runSceneRank);
                 break;
             }
+            case OPT_SCENETEST:
+            {
+                m_sceneTest = true;
+                break;
+            }
             case OPT_LOGLEVEL:
             {
                 LogLevel logLevel;
@@ -342,25 +353,6 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
                 m_language = language;
                 break;
             }
-            case OPT_DATADIR:
-            {
-                m_dataPath = optarg;
-                m_customDataPath = true;
-                GetLogger()->Info("Using custom datadir or running mod: '%s'\n", m_dataPath.c_str());
-                break;
-            }
-            case OPT_LANGDIR:
-            {
-                m_langPath = optarg;
-                GetLogger()->Info("Using custom language dir: '%s'\n", m_langPath.c_str());
-                break;
-            }
-            case OPT_TEXPACK:
-            {
-                m_texPackPath = optarg;
-                GetLogger()->Info("Using texturepack: '%s'\n", m_texPackPath.c_str());
-                break;
-            }
             case OPT_VBO:
             {
                 std::string vbo;
@@ -379,9 +371,49 @@ ParseArgsStatus CApplication::ParseArguments(int argc, char *argv[])
 
                 break;
             }
-            case OPT_PROTO:
+            case OPT_DATADIR:
             {
-                m_protoMode = true;
+                m_pathManager->SetDataPath(optarg);
+                GetLogger()->Info("Using data dir: '%s'\n", optarg);
+                break;
+            }
+            case OPT_LANGDIR:
+            {
+                m_pathManager->SetLangPath(optarg);
+                GetLogger()->Info("Using language dir: '%s'\n", optarg);
+                break;
+            }
+            case OPT_SAVEDIR:
+            {
+                m_pathManager->SetSavePath(optarg);
+                GetLogger()->Info("Using save dir: '%s'\n", optarg);
+                break;
+            }
+            case OPT_MOD:
+            {
+                m_pathManager->AddMod(optarg);
+                break;
+            }
+            case OPT_RESOLUTION:
+            {
+                std::istringstream resolution(optarg);
+                std::string w, h;
+                std::getline(resolution, w, 'x');
+                std::getline(resolution, h, 'x');
+
+                m_deviceConfig.size.x = atoi(w.c_str());
+                m_deviceConfig.size.y = atoi(h.c_str());
+                m_resolutionOverride = true;
+                break;
+            }
+            case OPT_HEADLESS:
+            {
+                m_headless = true;
+                break;
+            }
+            case OPT_DEVICE:
+            {
+                m_graphics = optarg;
                 break;
             }
             default:
@@ -399,66 +431,46 @@ bool CApplication::Create()
 
     GetLogger()->Info("Creating CApplication\n");
 
-    if (!GetProfile().InitCurrentDirectory())
+    m_errorMessage = m_pathManager->VerifyPaths();
+    if(!m_errorMessage.empty()) {
+        m_exitCode = 1;
+        return false;
+    }
+    m_pathManager->InitPaths();
+
+    if (!GetProfile().Init())
     {
         GetLogger()->Warn("Config not found. Default values will be used!\n");
         defaultValues = true;
     }
-    else
-    {
-        if (!m_customDataPath && GetProfile().GetLocalProfileString("Resources", "Data", path))
-            m_dataPath = path;
-    }
 
-    boost::filesystem::path dataPath(m_dataPath);
-    if (! (boost::filesystem::exists(dataPath) && boost::filesystem::is_directory(dataPath)) )
-    {
-        GetLogger()->Error("Data directory '%s' doesn't exist or is not a directory\n", m_dataPath.c_str());
-        m_errorMessage = std::string("Could not read from data directory:\n") +
-                         std::string("'") + m_dataPath + std::string("'\n") +
-                         std::string("Please check your installation, or supply a valid data directory by -datadir option.");
-        m_exitCode = 1;
-        return false;
+    if (GetProfile().GetStringProperty("Language", "Lang", path)) {
+        Language language;
+        if (ParseLanguage(path, language)) {
+            m_language = language;
+            GetLogger()->Info("Setting language '%s' from ini file\n", path.c_str());
+        } else {
+            GetLogger()->Error("Invalid language '%s' in ini file\n", path.c_str());
+        }
     }
 
-    GetProfile().SetLocalProfileString("Resources", "Data", m_dataPath);
-
     SetLanguage(m_language);
 
     //Create the sound instance.
     #ifdef OPENAL_SOUND
-    m_sound = static_cast<CSoundInterface *>(new ALSound());
+    if(!m_headless) {
+        m_sound = static_cast<CSoundInterface *>(new ALSound());
+    } else {
+        m_sound = new CSoundInterface();
+    }
     #else
     GetLogger()->Info("No sound support.\n");
     m_sound = new CSoundInterface();
     #endif
 
-    m_sound->Create(true);
-
-    // Cache sound files
-    if (defaultValues)
-    {
-        GetProfile().SetLocalProfileString("Resources", "Sound", GetDataSubdirPath(DIR_SOUND));
-        GetProfile().SetLocalProfileString("Resources", "Music", GetDataSubdirPath(DIR_MUSIC));
-    }
-
-    if (!m_customDataPath && GetProfile().GetLocalProfileString("Resources", "Sound", path))
-    {
-        m_sound->CacheAll(path);
-    }
-    else
-    {
-        m_sound->CacheAll(GetDataSubdirPath(DIR_SOUND));
-    }
-
-    if (!m_customDataPath && GetProfile().GetLocalProfileString("Resources", "Music", path))
-    {
-        m_sound->AddMusicFiles(path);
-    }
-    else
-    {
-        m_sound->AddMusicFiles(GetDataSubdirPath(DIR_MUSIC));
-    }
+    m_sound->Create();
+    m_sound->CacheAll();
+    m_sound->AddMusicFiles();
 
     GetLogger()->Info("CApplication created successfully\n");
 
@@ -495,35 +507,53 @@ bool CApplication::Create()
         return false;
     }
 
-    // load settings from profile
-    int iValue;
-    if ( GetProfile().GetLocalProfileInt("Setup", "Resolution", iValue) )
-    {
-        std::vector<Math::IntPoint> modes;
-        GetVideoResolutionList(modes, true, true);
-        if (static_cast<unsigned int>(iValue) < modes.size())
-            m_deviceConfig.size = modes.at(iValue);
+    if(!m_headless) {
+        // load settings from profile
+        int iValue;
+        std::string sValue;
+        if ( GetProfile().GetStringProperty("Setup", "Resolution", sValue) && !m_resolutionOverride )
+        {
+            std::istringstream resolution(sValue);
+            std::string ws, hs;
+            std::getline(resolution, ws, 'x');
+            std::getline(resolution, hs, 'x');
+            int w = 800, h = 600;
+            if(!ws.empty() && !hs.empty()) {
+                w = atoi(ws.c_str());
+                h = atoi(hs.c_str());
+            }
+
+            // Why not just set m_deviceConfig.size to w,h? Because this way if the resolution is no longer supported (e.g. changimg monitor) defaults will be used instead
+            std::vector<Math::IntPoint> modes;
+            GetVideoResolutionList(modes, true, true);
+            for(auto it = modes.begin(); it != modes.end(); ++it) {
+                if(it->x == w && it->y == h) {
+                    m_deviceConfig.size = *it;
+                    break;
+                }
+            }
+        }
+
+        if ( GetProfile().GetIntProperty("Setup", "Fullscreen", iValue) && !m_resolutionOverride )
+        {
+            m_deviceConfig.fullScreen = (iValue == 1);
+        }
+
+        if (! CreateVideoSurface())
+            return false; // dialog is in function
+
+        if (m_private->surface == nullptr)
+        {
+            m_errorMessage = std::string("SDL error while setting video mode:\n") +
+                            std::string(SDL_GetError());
+            GetLogger()->Error(m_errorMessage.c_str());
+            m_exitCode = 4;
+            return false;
+        }
+
+        SDL_WM_SetCaption(m_windowTitle.c_str(), m_windowTitle.c_str());
     }
 
-    if ( GetProfile().GetLocalProfileInt("Setup", "Fullscreen", iValue) )
-    {
-        m_deviceConfig.fullScreen = (iValue == 1);
-    }
-
-    if (! CreateVideoSurface())
-        return false; // dialog is in function
-
-    if (m_private->surface == nullptr)
-    {
-        m_errorMessage = std::string("SDL error while setting video mode:\n") +
-                         std::string(SDL_GetError());
-        GetLogger()->Error(m_errorMessage.c_str());
-        m_exitCode = 4;
-        return false;
-    }
-
-    SDL_WM_SetCaption(m_windowTitle.c_str(), m_windowTitle.c_str());
-
     // Enable translating key codes of key press events to unicode chars
     SDL_EnableUNICODE(1);
     SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
@@ -531,8 +561,26 @@ bool CApplication::Create()
     // Don't generate joystick events
     SDL_JoystickEventState(SDL_IGNORE);
 
-    // The video is ready, we can create and initalize the graphics device
-    m_device = new Gfx::CGLDevice(m_deviceConfig);
+    if(!m_headless)
+    {
+        // The video is ready, we can create and initalize the graphics device
+        if (m_graphics == "opengl")
+            m_device = new Gfx::CGLDevice(m_deviceConfig);
+        else if (m_graphics == "gl21")
+            m_device = new Gfx::CGL21Device(m_deviceConfig);
+        else if (m_graphics == "gl33")
+            m_device = new Gfx::CGL33Device(m_deviceConfig);
+        else
+        {
+            m_device = new Gfx::CNullDevice();
+            GetLogger()->Error("Unknown graphics device: %s\n", m_graphics.c_str());
+        }
+    }
+    else
+    {
+        m_device = new Gfx::CNullDevice();
+    }
+
     if (! m_device->Create() )
     {
         m_errorMessage = std::string("Error in CDevice::Create()\n") + standardInfoMessage;
@@ -556,14 +604,15 @@ bool CApplication::Create()
     m_modelManager = new Gfx::CModelManager(m_engine);
 
     // Create the robot application.
-    m_robotMain = new CRobotMain(this, !defaultValues);
+    m_controller = new CController(this, !defaultValues);
 
-    if (defaultValues) m_robotMain->CreateIni();
-
-    if (! m_runSceneName.empty())
-        m_robotMain->LoadSceneOnStart(m_runSceneName, m_runSceneRank);
-    else
-        m_robotMain->ChangePhase(PHASE_WELCOME1);
+    if (m_runSceneName.empty())
+        m_controller->StartApp();
+    else {
+        m_controller->GetRobotMain()->ChangePhase(PHASE_USER); // To load userlevel list - TODO: this is ugly
+        m_controller->GetRobotMain()->SetExitAfterMission(true);
+        m_controller->StartGame(m_runSceneName, m_runSceneRank/100, m_runSceneRank%100);
+    }
 
     return true;
 }
@@ -593,8 +642,8 @@ bool CApplication::CreateVideoSurface()
     if (m_deviceConfig.fullScreen)
         videoFlags |= SDL_FULLSCREEN;
 
-    if (m_deviceConfig.resizeable)
-        videoFlags |= SDL_RESIZABLE;
+    //if (m_deviceConfig.resizeable)
+    //    videoFlags |= SDL_RESIZABLE;
 
     // Set OpenGL attributes
 
@@ -623,23 +672,14 @@ void CApplication::Destroy()
 {
     m_joystickEnabled = false;
 
-    if (m_robotMain != nullptr)
-    {
-        delete m_robotMain;
-        m_robotMain = nullptr;
-    }
+    delete m_controller;
+    m_controller = nullptr;
 
-    if (m_sound != nullptr)
-    {
-        delete m_sound;
-        m_sound = nullptr;
-    }
+    delete m_sound;
+    m_sound = nullptr;
 
-    if (m_modelManager != nullptr)
-    {
-        delete m_modelManager;
-        m_modelManager = nullptr;
-    }
+    delete m_modelManager;
+    m_modelManager = nullptr;
 
     if (m_engine != nullptr)
     {
@@ -674,6 +714,17 @@ void CApplication::Destroy()
     SDL_Quit();
 }
 
+void CApplication::Restart()
+{
+    m_restart = true;
+    m_eventQueue->AddEvent(Event(EVENT_SYS_QUIT));
+}
+
+bool CApplication::IsRestarting()
+{
+    return m_restart;
+}
+
 bool CApplication::ChangeVideoConfig(const Gfx::GLDeviceConfig &newConfig)
 {
     static bool restore = false;
@@ -699,7 +750,7 @@ bool CApplication::ChangeVideoConfig(const Gfx::GLDeviceConfig &newConfig)
                           std::string(SDL_GetError()) + std::string("\n") +
                           std::string("Previous mode will be restored");
             GetLogger()->Error(error.c_str());
-            GetSystemUtils()->SystemDialog( SDT_ERROR, "COLOBT - Error", error);
+            GetSystemUtils()->SystemDialog( SDT_ERROR, "COLOBOT - Error", error);
 
             restore = true;
             ChangeVideoConfig(m_lastDeviceConfig);
@@ -712,7 +763,7 @@ bool CApplication::ChangeVideoConfig(const Gfx::GLDeviceConfig &newConfig)
             std::string error = std::string("SDL error while restoring previous video mode:\n") +
                           std::string(SDL_GetError());
             GetLogger()->Error(error.c_str());
-            GetSystemUtils()->SystemDialog( SDT_ERROR, "COLOBT - Fatal Error", error);
+            GetSystemUtils()->SystemDialog( SDT_ERROR, "COLOBOT - Fatal Error", error);
 
 
             // Fatal error, so post the quit event
@@ -724,6 +775,7 @@ bool CApplication::ChangeVideoConfig(const Gfx::GLDeviceConfig &newConfig)
     ( static_cast<Gfx::CGLDevice*>(m_device) )->ConfigChanged(m_deviceConfig);
 
     m_engine->ResetAfterDeviceChanged();
+    m_controller->GetRobotMain()->ResetAfterDeviceChanged();
 
     return true;
 }
@@ -840,7 +892,7 @@ void CApplication::UpdateMouse()
 {
     Math::IntPoint pos;
     SDL_GetMouseState(&pos.x, &pos.y);
-    m_mousePos = m_engine->WindowToInterfaceCoords(pos);
+    m_input->MouseMove(pos);
 }
 
 int CApplication::Run()
@@ -941,8 +993,8 @@ int CApplication::Run()
                 if (m_engine != nullptr)
                     passOn = m_engine->ProcessEvent(event);
 
-                if (passOn && m_robotMain != nullptr)
-                    m_robotMain->ProcessEvent(event);
+                if (passOn && m_controller != nullptr)
+                    m_controller->ProcessEvent(event);
             }
 
             StopPerformanceCounter(PCNT_EVENT_PROCESSING);
@@ -951,19 +1003,19 @@ int CApplication::Run()
 
             // Prepare and process step simulation event
             event = CreateUpdateEvent();
-            if (event.type != EVENT_NULL && m_robotMain != nullptr)
+            if (event.type != EVENT_NULL && m_controller != nullptr)
             {
                 LogEvent(event);
 
-                StartPerformanceCounter(PCNT_UPDATE_ENGINE);
-                m_engine->FrameUpdate();
-                StopPerformanceCounter(PCNT_UPDATE_ENGINE);
-
                 m_sound->FrameMove(m_relTime);
 
                 StartPerformanceCounter(PCNT_UPDATE_GAME);
-                m_robotMain->ProcessEvent(event);
+                m_controller->ProcessEvent(event);
                 StopPerformanceCounter(PCNT_UPDATE_GAME);
+
+                StartPerformanceCounter(PCNT_UPDATE_ENGINE);
+                m_engine->FrameUpdate();
+                StopPerformanceCounter(PCNT_UPDATE_ENGINE);
             }
 
             StopPerformanceCounter(PCNT_UPDATE_ALL);
@@ -982,7 +1034,7 @@ int CApplication::Run()
 
             if (m_lowCPU)
             {
-                usleep(20000); // should still give plenty of fps
+                GetSystemUtils()->Usleep(20000); // should still give plenty of fps
             }
         }
     }
@@ -1032,9 +1084,19 @@ Event CApplication::ProcessSystemEvent()
         event.key.virt = false;
         event.key.key = m_private->currentEvent.key.keysym.sym;
         event.key.unicode = m_private->currentEvent.key.keysym.unicode;
+        event.kmodState = m_private->currentEvent.key.keysym.mod;
 
-        // Use the occasion to update kmods
-        m_kmodState = m_private->currentEvent.key.keysym.mod;
+        // Some keyboards return numerical enter keycode instead of normal enter
+        // See issue #427 for details
+        if(event.key.key == KEY(KP_ENTER))
+            event.key.key = KEY(RETURN);
+
+        if(event.key.key == KEY(TAB) && ((event.kmodState & KEY_MOD(ALT)) != 0))
+        {
+            GetLogger()->Debug("Minimize to taskbar\n");
+            SDL_WM_IconifyWindow();
+            event.type = EVENT_NULL;
+        }
     }
     else if ( (m_private->currentEvent.type == SDL_MOUSEBUTTONDOWN) ||
          (m_private->currentEvent.type == SDL_MOUSEBUTTONUP) )
@@ -1059,24 +1121,13 @@ Event CApplication::ProcessSystemEvent()
                 event.type = EVENT_MOUSE_BUTTON_UP;
 
             event.mouseButton.button = static_cast<MouseButton>(1 << m_private->currentEvent.button.button);
-
-            // Use the occasion to update mouse button state
-            if (m_private->currentEvent.type == SDL_MOUSEBUTTONDOWN)
-                m_mouseButtonsState |= event.mouseButton.button;
-            else
-                m_mouseButtonsState &= ~event.mouseButton.button;
         }
-
-        // Use the occasion to update mouse pos
-        m_mousePos = m_engine->WindowToInterfaceCoords(
-            Math::IntPoint(m_private->currentEvent.button.x, m_private->currentEvent.button.y));
     }
     else if (m_private->currentEvent.type == SDL_MOUSEMOTION)
     {
         event.type = EVENT_MOUSE_MOVE;
 
-        m_mousePos = m_engine->WindowToInterfaceCoords(
-            Math::IntPoint(m_private->currentEvent.button.x, m_private->currentEvent.button.y));
+        m_input->MouseMove(Math::IntPoint(m_private->currentEvent.button.x, m_private->currentEvent.button.y));
     }
     else if (m_private->currentEvent.type == SDL_JOYAXISMOTION)
     {
@@ -1109,50 +1160,7 @@ Event CApplication::ProcessSystemEvent()
         event.active.gain = m_private->currentEvent.active.gain == 1;
     }
 
-
-    if (event.type == EVENT_KEY_DOWN)
-    {
-        if      (event.key.key == KEY(KP8))
-            m_trackedKeys |= TRKEY_NUM_UP;
-        else if (event.key.key == KEY(KP2))
-            m_trackedKeys |= TRKEY_NUM_DOWN;
-        else if (event.key.key == KEY(KP4))
-            m_trackedKeys |= TRKEY_NUM_LEFT;
-        else if (event.key.key == KEY(KP6))
-            m_trackedKeys |= TRKEY_NUM_RIGHT;
-        else if (event.key.key == KEY(KP_PLUS))
-            m_trackedKeys |= TRKEY_NUM_PLUS;
-        else if (event.key.key == KEY(KP_MINUS))
-            m_trackedKeys |= TRKEY_NUM_MINUS;
-        else if (event.key.key == KEY(PAGEUP))
-            m_trackedKeys |= TRKEY_PAGE_UP;
-        else if (event.key.key == KEY(PAGEDOWN))
-            m_trackedKeys |= TRKEY_PAGE_DOWN;
-    }
-    else if (event.type == EVENT_KEY_UP)
-    {
-        if      (event.key.key == KEY(KP8))
-            m_trackedKeys &= ~TRKEY_NUM_UP;
-        else if (event.key.key == KEY(KP2))
-            m_trackedKeys &= ~TRKEY_NUM_DOWN;
-        else if (event.key.key == KEY(KP4))
-            m_trackedKeys &= ~TRKEY_NUM_LEFT;
-        else if (event.key.key == KEY(KP6))
-            m_trackedKeys &= ~TRKEY_NUM_RIGHT;
-        else if (event.key.key == KEY(KP_PLUS))
-            m_trackedKeys &= ~TRKEY_NUM_PLUS;
-        else if (event.key.key == KEY(KP_MINUS))
-            m_trackedKeys &= ~TRKEY_NUM_MINUS;
-        else if (event.key.key == KEY(PAGEUP))
-            m_trackedKeys &= ~TRKEY_PAGE_UP;
-        else if (event.key.key == KEY(PAGEDOWN))
-            m_trackedKeys &= ~TRKEY_PAGE_DOWN;
-    }
-
-    event.trackedKeysState = m_trackedKeys;
-    event.kmodState = m_kmodState;
-    event.mousePos = m_mousePos;
-    event.mouseButtonsState = m_mouseButtonsState;
+    m_input->EventProcess(event);
 
     return event;
 }
@@ -1165,7 +1173,6 @@ void CApplication::LogEvent(const Event &event)
     {
         l->Trace(" rTime = %f\n", event.rTime);
         l->Trace(" kmodState = %04x\n", event.kmodState);
-        l->Trace(" trackedKeysState = %04x\n", event.trackedKeysState);
         l->Trace(" mousePos = %f, %f\n", event.mousePos.x, event.mousePos.y);
         l->Trace(" mouseButtonsState = %02x\n", event.mouseButtonsState);
         l->Trace(" customParam = %d\n", event.customParam);
@@ -1230,17 +1237,10 @@ Event CApplication::CreateVirtualEvent(const Event& sourceEvent)
     {
         virtualEvent.type = sourceEvent.type;
         virtualEvent.key = sourceEvent.key;
+        virtualEvent.key.key = GetVirtualKey(sourceEvent.key.key);
         virtualEvent.key.virt = true;
 
-        if (sourceEvent.key.key == KEY(LCTRL) || sourceEvent.key.key == KEY(RCTRL))
-            virtualEvent.key.key = VIRTUAL_KMOD(CTRL);
-        else if (sourceEvent.key.key == KEY(LSHIFT) || sourceEvent.key.key == KEY(RSHIFT))
-            virtualEvent.key.key = VIRTUAL_KMOD(SHIFT);
-        else if (sourceEvent.key.key == KEY(LALT) || sourceEvent.key.key == KEY(RALT))
-            virtualEvent.key.key = VIRTUAL_KMOD(ALT);
-        else if (sourceEvent.key.key == KEY(LMETA) || sourceEvent.key.key == KEY(RMETA))
-            virtualEvent.key.key = VIRTUAL_KMOD(META);
-        else
+        if(virtualEvent.key.key == sourceEvent.key.key)
             virtualEvent.type = EVENT_NULL;
     }
     else if ((sourceEvent.type == EVENT_JOY_BUTTON_DOWN) || (sourceEvent.type == EVENT_JOY_BUTTON_UP))
@@ -1259,6 +1259,8 @@ Event CApplication::CreateVirtualEvent(const Event& sourceEvent)
         virtualEvent.type = EVENT_NULL;
     }
 
+    m_input->EventProcess(virtualEvent);
+
     return virtualEvent;
 }
 
@@ -1348,10 +1350,7 @@ Event CApplication::CreateUpdateEvent()
     }
 
     Event frameEvent(EVENT_FRAME);
-    frameEvent.trackedKeysState = m_trackedKeys;
-    frameEvent.kmodState = m_kmodState;
-    frameEvent.mousePos = m_mousePos;
-    frameEvent.mouseButtonsState = m_mouseButtonsState;
+    m_input->EventProcess(frameEvent);
     frameEvent.rTime = m_relTime;
 
     return frameEvent;
@@ -1491,34 +1490,6 @@ bool CApplication::ParseDebugModes(const std::string& str, int& debugModes)
     return true;
 }
 
-int CApplication::GetKmods() const
-{
-    return m_kmodState;
-}
-
-bool CApplication::GetKmodState(int kmod) const
-{
-    return (m_kmodState & kmod) != 0;
-}
-
-bool CApplication::GetTrackedKeyState(TrackedKey key) const
-{
-    return (m_trackedKeys & key) != 0;
-}
-
-bool CApplication::GetMouseButtonState(int index) const
-{
-    return (m_mouseButtonsState & (1<<index)) != 0;
-}
-
-void CApplication::ResetKeyStates()
-{
-    GetLogger()->Trace("Reset key states\n");
-    m_trackedKeys = 0;
-    m_kmodState = 0;
-    m_robotMain->ResetKeyStates();
-}
-
 void CApplication::SetGrabInput(bool grab)
 {
     SDL_WM_GrabInput(grab ? SDL_GRAB_ON : SDL_GRAB_OFF);
@@ -1544,16 +1515,10 @@ MouseMode CApplication::GetMouseMode() const
     return m_mouseMode;
 }
 
-Math::Point CApplication::GetMousePos() const
-{
-    return m_mousePos;
-}
-
 void CApplication::MoveMouse(Math::Point pos)
 {
-    m_mousePos = pos;
-
     Math::IntPoint windowPos = m_engine->InterfaceToWindowCoords(pos);
+    m_input->MouseMove(windowPos);
     SDL_WarpMouse(windowPos.x, windowPos.y);
 }
 
@@ -1601,59 +1566,6 @@ bool CApplication::GetJoystickEnabled() const
     return m_joystickEnabled;
 }
 
-std::string CApplication::GetDataDirPath() const
-{
-    return m_dataPath;
-}
-
-std::string CApplication::GetDataSubdirPath(DataDir stdDir) const
-{
-    int index = static_cast<int>(stdDir);
-    assert(index >= 0 && index < DIR_MAX);
-    std::stringstream str;
-    str << m_dataPath;
-    str << "/";
-    str << m_standardDataDirs[index];
-    return str.str();
-}
-
-std::string CApplication::GetDataFilePath(DataDir stdDir, const std::string& subpath) const
-{
-    int index = static_cast<int>(stdDir);
-    assert(index >= 0 && index < DIR_MAX);
-    std::stringstream str;
-    str << m_dataPath;
-    str << "/";
-    str << m_standardDataDirs[index];
-    if (stdDir == DIR_HELP)
-    {
-        str << "/";
-        str << GetLanguageChar();
-    }
-    str << "/";
-    str << subpath;
-    return str.str();
-}
-
-std::string CApplication::GetTexPackFilePath(const std::string& textureName) const
-{
-    std::stringstream str;
-
-    if (! m_texPackPath.empty())
-    {
-        str << m_texPackPath;
-        str << "/";
-        str << textureName;
-        if (! boost::filesystem::exists(str.str()))
-        {
-            GetLogger()->Trace("Texture '%s' not in texpack\n", textureName.c_str());
-            str.str("");
-        }
-    }
-
-    return str.str();
-}
-
 Language CApplication::GetLanguage() const
 {
     return m_language;
@@ -1681,6 +1593,10 @@ char CApplication::GetLanguageChar() const
         case LANGUAGE_POLISH:
             langChar = 'P';
             break;
+
+        case LANGUAGE_RUSSIAN:
+            langChar = 'R';
+            break;
     }
     return langChar;
 }
@@ -1707,6 +1623,11 @@ bool CApplication::ParseLanguage(const std::string& str, Language& language)
         language = LANGUAGE_POLISH;
         return true;
     }
+    else if (str == "ru")
+    {
+        language = LANGUAGE_RUSSIAN;
+        return true;
+    }
 
     return false;
 }
@@ -1740,6 +1661,10 @@ void CApplication::SetLanguage(Language language)
         case LANGUAGE_POLISH:
             locale = "pl_PL.utf8";
             break;
+
+        case LANGUAGE_RUSSIAN:
+            locale = "ru_RU.utf8";
+            break;
     }
 
     if (locale.empty())
@@ -1770,6 +1695,10 @@ void CApplication::SetLanguage(Language language)
             {
                 m_language = LANGUAGE_POLISH;
             }
+            else if (strncmp(envLang,"ru",2) == 0)
+            {
+                m_language = LANGUAGE_RUSSIAN;
+            }
             else
             {
                 GetLogger()->Warn("Enviromnent locale ('%s') is not supported, setting default language\n", envLang);
@@ -1786,9 +1715,34 @@ void CApplication::SetLanguage(Language language)
         GetLogger()->Trace("SetLanguage: Set LANGUAGE=%s in environment\n", locale.c_str());
     }
 
-    setlocale(LC_ALL, "");
+    char* defaultLocale = setlocale(LC_ALL, ""); // Load system locale
+    setlocale(LC_NUMERIC, "C"); // Force numeric locale to "C" (fixes decimal point problems)
+    char* systemLocale = setlocale(LC_ALL, nullptr); // Get current locale configuration
+    GetLogger()->Debug("Default system locale: %s\n", defaultLocale);
+    GetLogger()->Debug("Setting locale: %s\n", systemLocale);
+    // Update C++ locale
+    try
+    {
+        std::locale::global(std::locale(systemLocale));
+    }
+    catch(...)
+    {
+        GetLogger()->Warn("Failed to update locale, possibly incorect system configuration. Will fallback to classic locale.\n");
+        try
+        {
+            std::locale::global(std::locale::classic());
+        }
+        catch(...)
+        {
+            GetLogger()->Warn("Failed to set classic locale. Something is really messed up in your system configuration. Translations might not work.\n");
+        }
 
-    bindtextdomain("colobot", m_langPath.c_str());
+        // C locale might still work correctly
+        setlocale(LC_ALL, "");
+        setlocale(LC_NUMERIC, "C");
+    }
+
+    bindtextdomain("colobot", m_pathManager->GetLangPath().c_str());
     bind_textdomain_codeset("colobot", "UTF-8");
     textdomain("colobot");
 
@@ -1844,8 +1798,7 @@ void CApplication::UpdatePerformanceCountersData()
     }
 }
 
-bool CApplication::GetProtoMode() const
+bool CApplication::GetSceneTestMode()
 {
-    return m_protoMode;
+    return m_sceneTest;
 }
-
