@@ -1,34 +1,39 @@
-// * This file is part of the COLOBOT source code
-// * Copyright (C) 2001-2008, Daniel ROUX & EPSITEC SA, www.epsitec.ch
-// * Copyright (C) 2012 Polish Portal of Colobot (PPC)
-// *
-// * This program is free software: you can redistribute it and/or modify
-// * it under the terms of the GNU General Public License as published by
-// * the Free Software Foundation, either version 3 of the License, or
-// * (at your option) any later version.
-// *
-// * This program is distributed in the hope that it will be useful,
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// * GNU General Public License for more details.
-// *
-// * You should have received a copy of the GNU General Public License
-// * along with this program. If not, see  http://www.gnu.org/licenses/.
+/*
+ * This file is part of the Colobot: Gold Edition source code
+ * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see http://gnu.org/licenses
+ */
 
 
 #include "object/auto/autofactory.h"
 
 #include "common/global.h"
-#include "common/iman.h"
 
 #include "math/geometry.h"
 
-#include "object/robotmain.h"
 #include "object/brain.h"
+#include "object/objman.h"
+#include "object/robotmain.h"
+#include "object/level/parserline.h"
+#include "object/level/parserparam.h"
 
 #include "physics/physics.h"
 
 #include "script/cmdtoken.h"
+#include "script/script.h"
 
 #include "ui/interface.h"
 #include "ui/window.h"
@@ -492,9 +497,9 @@ bool CAutoFactory::EventProcess(const Event &event)
                 CBrain* brain = m_vehicle->GetBrain();
                 if ( brain != nullptr )
                 {
-                    brain->SendProgram(0, const_cast<const char*>(m_program));
-                    brain->SetScriptRun(0);
-                    brain->RunProgram(0);
+                    Program* program = brain->AddProgram();
+                    program->script->SendScript(const_cast<const char*>(m_program));
+                    brain->RunProgram(program);
                 }
             }
 
@@ -513,40 +518,29 @@ bool CAutoFactory::EventProcess(const Event &event)
 
 // Saves all parameters of the controller.
 
-bool CAutoFactory::Write(char *line)
+bool CAutoFactory::Write(CLevelParserLine* line)
 {
-    char    name[100];
-
     if ( m_phase == AFP_WAIT )  return false;
-
-    sprintf(name, " aExist=%d", 1);
-    strcat(line, name);
-
+    
+    line->AddParam("aExist", new CLevelParserParam(true));
     CAuto::Write(line);
-
-    sprintf(name, " aPhase=%d", m_phase);
-    strcat(line, name);
-
-    sprintf(name, " aProgress=%.2f", m_progress);
-    strcat(line, name);
-
-    sprintf(name, " aSpeed=%.2f", m_speed);
-    strcat(line, name);
+    line->AddParam("aPhase", new CLevelParserParam(static_cast<int>(m_phase)));
+    line->AddParam("aProgress", new CLevelParserParam(m_progress));
+    line->AddParam("aSpeed", new CLevelParserParam(m_speed));
 
     return true;
 }
 
 // Restores all parameters of the controller
 
-bool CAutoFactory::Read(char *line)
+bool CAutoFactory::Read(CLevelParserLine* line)
 {
-    if ( OpInt(line, "aExist", 0) == 0 )  return false;
+    if ( !line->GetParam("aExist")->AsBool(false) )  return false;
 
     CAuto::Read(line);
-
-    m_phase = static_cast< AutoFactoryPhase >(OpInt(line, "aPhase", AFP_WAIT));
-    m_progress = OpFloat(line, "aProgress", 0.0f);
-    m_speed = OpFloat(line, "aSpeed", 1.0f);
+    m_phase = static_cast< AutoFactoryPhase >(line->GetParam("aPhase")->AsInt(AFP_WAIT));
+    m_progress = line->GetParam("aProgress")->AsFloat(0.0f);
+    m_speed = line->GetParam("aSpeed")->AsFloat(1.0f);
 
     m_lastParticle = 0.0f;
     m_fretPos = m_object->GetPosition(0);
@@ -563,23 +557,21 @@ CObject* CAutoFactory::SearchFret()
     Math::Vector    oPos;
     ObjectType  type;
     float       dist;
-    int         i;
-
-    for ( i=0 ; i<1000000 ; i++ )
+    
+    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = static_cast< CObject* >(m_iMan->SearchInstance(CLASS_OBJECT, i));
-        if ( pObj == 0 )  break;
-
+        pObj = it.second;
+        
         type = pObj->GetType();
         if ( type != OBJECT_METAL )  continue;
         if ( pObj->GetTruck() != 0 )  continue;
-
+        
         oPos = pObj->GetPosition(0);
         dist = Math::Distance(oPos, m_fretPos);
-
+        
         if ( dist < 8.0f )  return pObj;
     }
-
+    
     return 0;
 }
 
@@ -591,14 +583,12 @@ bool CAutoFactory::NearestVehicle()
     Math::Vector    cPos, oPos;
     ObjectType  type;
     float       oRadius, dist;
-    int         i;
 
     cPos = m_object->GetPosition(0);
-
-    for ( i=0 ; i<1000000 ; i++ )
+    
+    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = static_cast< CObject* >(m_iMan->SearchInstance(CLASS_OBJECT, i));
-        if ( pObj == 0 )  break;
+        pObj = it.second;
 
         type = pObj->GetType();
         if ( type != OBJECT_HUMAN    &&
@@ -655,7 +645,6 @@ bool CAutoFactory::CreateVehicle()
     Math::Vector    pos;
     float       angle;
     char*       name;
-    int         i;
 
     angle = m_object->GetAngleY(0);
 
@@ -690,11 +679,17 @@ bool CAutoFactory::CreateVehicle()
         physics->SetFreeze(true);  // it doesn't move
     }
 
-    for ( i=0 ; i<10 ; i++ )
+    CBrain* brain = vehicle->GetBrain();
+    if(brain != nullptr)
     {
-        name = m_main->GetNewScriptName(m_type, i);
-        if ( name == 0 )  break;
-        vehicle->ReadProgram(i, name);
+        for ( int i=0 ; ; i++ )
+        {
+            name = m_main->GetNewScriptName(m_type, i);
+            if ( name == nullptr )  break;
+            Program* prog = brain->GetOrAddProgram(i);
+            vehicle->ReadProgram(prog, name);
+            prog->readOnly = true;
+        }
     }
 
     return true;
@@ -703,34 +698,31 @@ bool CAutoFactory::CreateVehicle()
 // Seeking the vehicle during manufacture.
 
 CObject* CAutoFactory::SearchVehicle()
-{
+{    
     CObject*    pObj;
     Math::Vector    oPos;
     ObjectType  type;
     float       dist;
-    int         i;
-
-    for ( i=0 ; i<1000000 ; i++ )
+    
+    for(auto it : CObjectManager::GetInstancePointer()->GetAllObjects())
     {
-        pObj = static_cast< CObject* >(m_iMan->SearchInstance(CLASS_OBJECT, i));
-        if ( pObj == 0 )  break;
-
+        pObj = it.second;
+        
         if ( !pObj->GetLock() )  continue;
-
+        
         type = pObj->GetType();
         if ( type != m_type )  continue;
         if ( pObj->GetTruck() != 0 )  continue;
-
+        
         oPos = pObj->GetPosition(0);
         dist = Math::Distance(oPos, m_fretPos);
-
+        
         if ( dist < 8.0f )  return pObj;
     }
-
+    
     return 0;
 }
 
-
 // Creates all the interface when the object is selected.
 
 bool CAutoFactory::CreateInterface(bool bSelect)
