@@ -1,19 +1,21 @@
-// * This file is part of the COLOBOT source code
-// * Copyright (C) 2001-2008, Daniel ROUX & EPSITEC SA, www.epsitec.ch
-// * Copyright (C) 2012, Polish Portal of Colobot (PPC)
-// *
-// * This program is free software: you can redistribute it and/or modify
-// * it under the terms of the GNU General Public License as published by
-// * the Free Software Foundation, either version 3 of the License, or
-// * (at your option) any later version.
-// *
-// * This program is distributed in the hope that it will be useful,
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// * GNU General Public License for more details.
-// *
-// * You should have received a copy of the GNU General Public License
-// * along with this program. If not, see  http://www.gnu.org/licenses/.
+/*
+ * This file is part of the Colobot: Gold Edition source code
+ * Copyright (C) 2001-2014, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * http://epsiteс.ch; http://colobot.info; http://github.com/colobot
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see http://gnu.org/licenses
+ */
 
 
 #include "ui/window.h"
@@ -64,34 +66,20 @@ CWindow::~CWindow()
 
 void CWindow::Flush()
 {
-    int     i;
-
-    for ( i=0 ; i<MAXWINDOW ; i++ )
+    for (int i = 0 ; i < MAXWINDOW; i++)
     {
-        if ( m_table[i] != 0 )
-        {
-            delete m_table[i];
-            m_table[i] = 0;
-        }
+        delete m_table[i];
+        m_table[i] = nullptr;
     }
 
-    if ( m_buttonReduce != 0 )
-    {
-        delete m_buttonReduce;
-        m_buttonReduce = 0;
-    }
+    delete m_buttonReduce;
+    m_buttonReduce = nullptr;
 
-    if ( m_buttonFull != 0 )
-    {
-        delete m_buttonFull;
-        m_buttonFull = 0;
-    }
+    delete m_buttonFull;
+    m_buttonFull = nullptr;
 
-    if ( m_buttonClose != 0 )
-    {
-        delete m_buttonClose;
-        m_buttonClose = 0;
-    }
+    delete m_buttonClose;
+    m_buttonClose = nullptr;
 }
 
 
@@ -583,23 +571,14 @@ void CWindow::SetName(std::string name, bool tooltip)
 
     CControl::SetName(name, tooltip);
 
-    if ( m_buttonReduce != 0 )
-    {
-        delete m_buttonReduce;
-        m_buttonReduce = 0;
-    }
+    delete m_buttonReduce;
+    m_buttonReduce = nullptr;
 
-    if ( m_buttonFull != 0 )
-    {
-        delete m_buttonFull;
-        m_buttonFull = 0;
-    }
+    delete m_buttonFull;
+    m_buttonFull = nullptr;
 
-    if ( m_buttonClose != 0 )
-    {
-        delete m_buttonClose;
-        m_buttonClose = 0;
-    }
+    delete m_buttonClose;
+    m_buttonClose = nullptr;
 
     bAdjust = false;
 
@@ -788,7 +767,7 @@ bool CWindow::GetFixed()
 
 void CWindow::AdjustButtons()
 {
-    char    res[100];
+    std::string res;
 
     if ( m_buttonFull != 0 )
     {
@@ -796,13 +775,13 @@ void CWindow::AdjustButtons()
         {
             m_buttonFull->SetIcon(54);
             GetResource(RES_TEXT, RT_WINDOW_STANDARD, res);
-            m_buttonFull->SetTooltip(std::string(res));
+            m_buttonFull->SetTooltip(res);
         }
         else
         {
             m_buttonFull->SetIcon(52);
             GetResource(RES_TEXT, RT_WINDOW_MAXIMIZED, res);
-            m_buttonFull->SetTooltip(std::string(res));
+            m_buttonFull->SetTooltip(res);
         }
     }
 
@@ -812,13 +791,13 @@ void CWindow::AdjustButtons()
         {
             m_buttonReduce->SetIcon(54);
             GetResource(RES_TEXT, RT_WINDOW_STANDARD, res);
-            m_buttonReduce->SetTooltip(std::string(res));
+            m_buttonReduce->SetTooltip(res);
         }
         else
         {
             m_buttonReduce->SetIcon(51);
             GetResource(RES_TEXT, RT_WINDOW_MINIMIZED, res);
-            m_buttonReduce->SetTooltip(std::string(res));
+            m_buttonReduce->SetTooltip(res);
         }
     }
 
@@ -826,7 +805,7 @@ void CWindow::AdjustButtons()
     {
         m_buttonClose->SetIcon(11);  // x
         GetResource(RES_TEXT, RT_WINDOW_CLOSE, res);
-        m_buttonClose->SetTooltip(std::string(res));
+        m_buttonClose->SetTooltip(res);
     }
 }
 
@@ -1174,7 +1153,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
 
     if ( icon == 0 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_WHITE);
         uv1.x =  64.0f/256.0f;  // dark blue transparent
         uv1.y =  64.0f/256.0f;
@@ -1190,9 +1169,9 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 1 )
     {
-        m_engine->SetTexture("button1.png");
-        m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_WHITE);
-        uv1.x = 128.0f/256.0f;  // yellow tooltip
+        m_engine->SetTexture("textures/interface/button1.png");
+        m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
+        uv1.x = 128.0f/256.0f;  // white tooltip
         uv1.y =   0.0f/256.0f;
         uv2.x = 224.0f/256.0f;
         uv2.y =  16.0f/256.0f;
@@ -1204,7 +1183,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 2 )
     {
-        m_engine->SetTexture("button1.png");
+        m_engine->SetTexture("textures/interface/button1.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x = 128.0f/256.0f;  // yellow
         uv1.y =  16.0f/256.0f;
@@ -1218,7 +1197,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 3 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_BLACK);
         uv1.x =   0.0f/256.0f;  // transparent blue bar with yellow upper
         uv1.y =  64.0f/256.0f;
@@ -1237,7 +1216,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
         dim.x += 100.0f/640.0f;
         dim.y +=  60.0f/480.0f;
 
-        m_engine->SetTexture("human.png");
+        m_engine->SetTexture("textures/objects/human.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x = 140.0f/256.0f;
         uv1.y =  32.0f/256.0f;
@@ -1254,7 +1233,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
         dim.x -= 20.0f/640.0f;
         dim.y +=  0.0f/480.0f;
 
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_WHITE);
         uv1.x = 192.0f/256.0f;
         uv1.y =  32.0f/256.0f;
@@ -1273,7 +1252,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
         dim.x -= 20.0f/640.0f;
         dim.y -= 20.0f/480.0f;
 
-        m_engine->SetTexture("button1.png");
+        m_engine->SetTexture("textures/interface/button1.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x =  64.0f/256.0f;
         uv1.y =   0.0f/256.0f;
@@ -1309,7 +1288,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
         dim.x -= 20.0f/640.0f;
         dim.y -= 20.0f/480.0f;
 
-        m_engine->SetTexture("button3.png");
+        m_engine->SetTexture("textures/interface/button3.png");
         uv1.x =   0.0f/256.0f;
         uv1.y = 224.0f/256.0f;
         uv2.x =  32.0f/256.0f;
@@ -1320,7 +1299,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
         uv2.y -= dp;
         DrawIcon(pos, dim, uv1, uv2);  // dark blue background
 
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         uv1.x = 224.0f/256.0f;
         uv1.y = 224.0f/256.0f;
         uv2.x = 249.0f/256.0f;
@@ -1392,7 +1371,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 5 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_BLACK);
         uv1.x =  64.0f/256.0f;  // transparent green
         uv1.y = 160.0f/256.0f;
@@ -1406,7 +1385,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 6 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_BLACK);
         uv1.x =  64.0f/256.0f;  // transparent red
         uv1.y = 176.0f/256.0f;
@@ -1420,7 +1399,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 7 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_BLACK);
         uv1.x =  64.0f/256.0f;  // transparent blue
         uv1.y = 192.0f/256.0f;
@@ -1434,7 +1413,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 8 )
     {
-        m_engine->SetTexture("button1.png");
+        m_engine->SetTexture("textures/interface/button1.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x =   0.0f/256.0f;  // opaque orange
         uv1.y =   0.0f/256.0f;
@@ -1450,7 +1429,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 9 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x =  32.0f/256.0f;  // opaque gray
         uv1.y =  32.0f/256.0f;
@@ -1470,7 +1449,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 11 )
     {
-        m_engine->SetTexture("button2.png");
+        m_engine->SetTexture("textures/interface/button2.png");
         m_engine->SetState(Gfx::ENG_RSTATE_TTEXTURE_BLACK);
         uv1.x =  64.0f/256.0f;  // transparent yellow
         uv1.y = 224.0f/256.0f;
@@ -1484,7 +1463,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 12 )
     {
-        m_engine->SetTexture("button1.png");
+        m_engine->SetTexture("textures/interface/button1.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x = 128.0f/256.0f;  // dirty opaque gray
         uv1.y = 128.0f/256.0f;
@@ -1500,7 +1479,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 13 )
     {
-        m_engine->SetTexture("button1.png");
+        m_engine->SetTexture("textures/interface/button1.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x = 192.0f/256.0f;  //  dirty opaque blue
         uv1.y = 128.0f/256.0f;
@@ -1516,7 +1495,7 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
     }
     else if ( icon == 14 )
     {
-        m_engine->SetTexture("button1.png");
+        m_engine->SetTexture("textures/interface/button1.png");
         m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
         uv1.x = 160.0f/256.0f;  // dirty opaque red
         uv1.y = 128.0f/256.0f;
@@ -1536,15 +1515,13 @@ void CWindow::DrawVertex(Math::Point pos, Math::Point dim, int icon)
 
 void CWindow::DrawHach(Math::Point pos, Math::Point dim)
 {
-#if _NEWLOOK
-#else
     Math::Point     ppos, ddim, uv1, uv2;
     float       dp, max, ndim;
     bool        bStop;
 
     dp = 0.5f/256.0f;
 
-    m_engine->SetTexture("button2.png");
+    m_engine->SetTexture("textures/interface/button2.png");
     m_engine->SetState(Gfx::ENG_RSTATE_NORMAL);
     uv1.x =  64.0f/256.0f;  // hatching
     uv1.y = 208.0f/256.0f;
@@ -1575,7 +1552,6 @@ void CWindow::DrawHach(Math::Point pos, Math::Point dim)
         ppos.x += ddim.x;
     }
     while ( !bStop );
-#endif
 }
 
 }
